@@ -105,7 +105,9 @@ __ATTRIB(__DRI_ATTRIB_BUFFER_SIZE, rgbBits),
         __ATTRIB(__DRI_ATTRIB_BIND_TO_TEXTURE_RGB, bindToTextureRgb),
         __ATTRIB(__DRI_ATTRIB_BIND_TO_TEXTURE_RGBA, bindToTextureRgba),
         __ATTRIB(__DRI_ATTRIB_BIND_TO_MIPMAP_TEXTURE, bindToMipmapTexture),
-        __ATTRIB(__DRI_ATTRIB_YINVERTED, yInverted),};
+        __ATTRIB(__DRI_ATTRIB_YINVERTED, yInverted),
+        __ATTRIB(__DRI_ATTRIB_FRAMEBUFFER_SRGB_CAPABLE, sRGBCapable),
+        };
 
 static void
 setScalar(__GLXconfig * config, unsigned int attrib, unsigned int value)
@@ -209,14 +211,6 @@ glxConvertConfigs(const __DRIcoreExtension * core,
 
 static const char dri_driver_path[] = DRI_DRIVER_PATH;
 
-/* Temporary define to allow building without a dri_interface.h from
- * updated Mesa.  Some day when we don't care about Mesa that old any
- * more this can be removed.
- */
-#ifndef __DRI_DRIVER_GET_EXTENSIONS
-#define __DRI_DRIVER_GET_EXTENSIONS "__driDriverGetExtensions"
-#endif
-
 void *
 glxProbeDriver(const char *driverName,
                void **coreExt, const char *coreName, int coreVersion,
@@ -225,8 +219,7 @@ glxProbeDriver(const char *driverName,
     int i;
     void *driver;
     char filename[PATH_MAX];
-    char *get_extensions_name;
-    const __DRIextension **extensions = NULL;
+    const __DRIextension **extensions;
 
     snprintf(filename, sizeof filename, "%s/%s_dri.so",
              dri_driver_path, driverName);
@@ -238,18 +231,7 @@ glxProbeDriver(const char *driverName,
         goto cleanup_failure;
     }
 
-    if (asprintf(&get_extensions_name, "%s_%s",
-                 __DRI_DRIVER_GET_EXTENSIONS, driverName) != -1) {
-        const __DRIextension **(*get_extensions)(void);
-
-        get_extensions = dlsym(driver, get_extensions_name);
-        if (get_extensions)
-            extensions = get_extensions();
-        free(get_extensions_name);
-    }
-
-    if (!extensions)
-        extensions = dlsym(driver, __DRI_DRIVER_EXTENSIONS);
+    extensions = dlsym(driver, __DRI_DRIVER_EXTENSIONS);
     if (extensions == NULL) {
         LogMessage(X_ERROR, "AIGLX error: %s exports no extensions (%s)\n",
                    driverName, dlerror());
